window.Gleam.initSearch([{"doc":"handles","title":"handles","content":"[![Package Version](https://img.shields.io/hexpm/v/handlebars)](https://hex.pm/packages/handlebars)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/handlebars/)\n\n# handles\n\n`handles` is a templating language written in pure Gleam. Heavily inspired by [mustache](https://mustache.github.io/) and [Handlebars.js](https://github.com/handlebars-lang/handlebars.js).\n\n```sh\ngleam add handles\n```\n\n```gleam\nimport handles\nimport gleam/io\n\npub fn main() {\n  let assert Ok(template) = handles.prepare(&quot;Hello {{name}}&quot;)\n  let assert Ok(string) =\n    handles.run(\n      template,\n      dict.new()\n        |&gt; dict.insert(&quot;name&quot;, &quot;Oliver&quot;)\n        |&gt; dynamic.from,\n    )\n\n  io.println(string)\n}\n```\n\n## Usage Documentation\n\n__Handles__ a is very simple templating language that consists of a single primitive, the &quot;tag&quot;.\nA tag starts with two open braces `{{`, followed by a string body, and ends with two closing braces `}}`.\nThere are two kinds of tags, [Property tags](#property-tags) and [Block tags](#block-tags).\n\n### Property tags\n\nA property tag is used to access properties passed into the template engine and insert them into the resulting string in-place of the property tag.\nValues accessed by a property tag must be of type `String`, `Int`, or `Float`, or it will result in a runtime error.\nAccessing a property which was not passed into the template engine will result in a runtime error.\n\n```handlebars\n{{some.property.path}}\n```\n\n### Block tags\n\nA block tag is used to temporarily alter the behavior of the templating engine.\nEach block tag have two variants; A start tag indicated by a leading `#` and a stop tag indicated by a leading `/`.\nA blocks start tag accepts a property accessor, while the end tag does not.\n\n#### if\n\n`if` blocks are used to conditionally _include_ parts of a templated based on the value of a property.\nValues accessed by an if block must be of type `Bool` or it will result in a runtime error.\nAccessing a property which was not passed into the template engine will result in a runtime error.\n\n```handlebars\n{{#if some.prop}}\n  {{name}}\n{{/if}}\n```\n\n#### unless\n\n`unless` blocks are used to conditionally _exclude_ parts of a templated based on the value of a property.\nValues accessed by an `unless` block must be of type `Bool` or it will result in a runtime error.\nAccessing a property which was not passed into the template engine will result in a runtime error.\n\n```handlebars\n{{#unless some.prop}}\n  {{name}}\n{{/unless}}\n```\n\n#### each\n\n`each` blocks are used to include a part of a templated zero or more times.\nValues accessed by an `each` block must be of type `List(Dict)` or it will result in a runtime error.\nAccessing a property which was not passed into the template engine will result in a runtime error.\nThe context of which properties are resolved while inside the each block will be scoped to the current item from the list.\n\n```handlebars\n{{#each some.prop}}\n  {{name}}\n{{/each}}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/handles&gt;.\n\n## Development\n\n```sh\ngleam test\n```\n","url":"index.html"},{"doc":"handles/lexer","title":"LexError","content":"pub type LexError {\n  UnbalancedTag(start: Int, end: Int)\n  SyntaxError(errors: List(SyntaxError))\n}\n\nUnbalancedTag(start: Int, end: Int)\n\n\nSyntaxError(errors: List(SyntaxError))\n\n\nSynonyms:\nhandles/lexer.LexError\nhandles/lexer LexError","url":"handles/lexer.html#LexError"},{"doc":"handles/lexer","title":"SyntaxError","content":"pub type SyntaxError {\n  MissingBody(start: Int, end: Int)\n  MissingBlockKind(start: Int, end: Int)\n  UnexpectedBlockArgument(start: Int, end: Int)\n}\n\nMissingBody(start: Int, end: Int)\n\n\nMissingBlockKind(start: Int, end: Int)\n\n\nUnexpectedBlockArgument(start: Int, end: Int)\n\n\nSynonyms:\nhandles/lexer.SyntaxError\nhandles/lexer SyntaxError","url":"handles/lexer.html#SyntaxError"},{"doc":"handles/lexer","title":"Token","content":"pub type Token {\n  Constant(start: Int, end: Int, value: String)\n  Property(start: Int, end: Int, path: List(String))\n  BlockStart(\n    start: Int,\n    end: Int,\n    kind: String,\n    path: List(String),\n  )\n  BlockEnd(start: Int, end: Int, kind: String)\n}\n\nConstant(start: Int, end: Int, value: String)\n\n\nProperty(start: Int, end: Int, path: List(String))\n\n\nBlockStart(\n  start: Int,\n  end: Int,\n  kind: String,\n  path: List(String),\n)\n\n\nBlockEnd(start: Int, end: Int, kind: String)\n\n\nSynonyms:\nhandles/lexer.Token\nhandles/lexer Token","url":"handles/lexer.html#Token"},{"doc":"handles/lexer","title":"run","content":"pub fn run(template: String) -&gt; Result(List(Token), LexError)\n\nSynonyms:\nhandles/lexer.run\nhandles/lexer run","url":"handles/lexer.html#run"},{"doc":"handles/lexer","title":"handles/lexer","content":"","url":"handles/lexer.html"},{"doc":"handles/parser","title":"AST","content":"pub type AST {\n  Constant(value: String)\n  Property(path: List(String))\n  Block(kind: String, path: List(String), children: List(AST))\n}\n\nConstant(value: String)\n\n\nProperty(path: List(String))\n\n\nBlock(kind: String, path: List(String), children: List(AST))\n\n\nSynonyms:\nhandles/parser.AST\nhandles/parser AST","url":"handles/parser.html#AST"},{"doc":"handles/parser","title":"ParseError","content":"pub type ParseError {\n  UnbalancedBlock(start: Int, end: Int, kind: String)\n  UnknownBlockKind(start: Int, end: Int, kind: String)\n}\n\nUnbalancedBlock(start: Int, end: Int, kind: String)\n\n\nUnknownBlockKind(start: Int, end: Int, kind: String)\n\n\nSynonyms:\nhandles/parser.ParseError\nhandles/parser ParseError","url":"handles/parser.html#ParseError"},{"doc":"handles/parser","title":"run","content":"pub fn run(\n  tokens: List(Token),\n  valid_blocks: List(String),\n) -&gt; Result(List(AST), List(ParseError))\n\nSynonyms:\nhandles/parser.run\nhandles/parser run","url":"handles/parser.html#run"},{"doc":"handles/parser","title":"handles/parser","content":"","url":"handles/parser.html"},{"doc":"handles/engine","title":"RuntimeError","content":"pub type RuntimeError {\n  UnexpectedTypeError(\n    path: List(String),\n    got: String,\n    expected: List(String),\n  )\n  UnknownPropertyError(path: List(String))\n}\n\nUnexpectedTypeError(\n  path: List(String),\n  got: String,\n  expected: List(String),\n)\n\n\nUnknownPropertyError(path: List(String))\n\n\nSynonyms:\nhandles/engine.RuntimeError\nhandles/engine RuntimeError","url":"handles/engine.html#RuntimeError"},{"doc":"handles/engine","title":"get_as_bool","content":"pub fn get_as_bool(\n  root_ctx: Dynamic,\n  path: List(String),\n) -&gt; Result(Bool, RuntimeError)\n\nSynonyms:\nhandles/engine.get_as_bool\nhandles/engine get_as_bool","url":"handles/engine.html#get_as_bool"},{"doc":"handles/engine","title":"get_as_list","content":"pub fn get_as_list(\n  root_ctx: Dynamic,\n  path: List(String),\n) -&gt; Result(List(Dynamic), RuntimeError)\n\nSynonyms:\nhandles/engine.get_as_list\nhandles/engine get_as_list","url":"handles/engine.html#get_as_list"},{"doc":"handles/engine","title":"get_as_string","content":"pub fn get_as_string(\n  root_ctx: Dynamic,\n  path: List(String),\n) -&gt; Result(String, RuntimeError)\n\nSynonyms:\nhandles/engine.get_as_string\nhandles/engine get_as_string","url":"handles/engine.html#get_as_string"},{"doc":"handles/engine","title":"run","content":"pub fn run(\n  ast: List(AST),\n  ctx: Dynamic,\n) -&gt; Result(String, RuntimeError)\n\nSynonyms:\nhandles/engine.run\nhandles/engine run","url":"handles/engine.html#run"},{"doc":"handles/engine","title":"handles/engine","content":"","url":"handles/engine.html"},{"doc":"handles","title":"HandlesError","content":"pub type HandlesError {\n  LexError(error: lexer.LexError)\n  ParseError(error: List(parser.ParseError))\n  RuntimeError(error: engine.RuntimeError)\n}\n\nLexError(error: lexer.LexError)\n\n\nParseError(error: List(parser.ParseError))\n\n\nRuntimeError(error: engine.RuntimeError)\n\n\nSynonyms:\nhandles.HandlesError\nhandles HandlesError","url":"handles.html#HandlesError"},{"doc":"handles","title":"Template","content":"pub type Template {\n  Template(ast: List(parser.AST))\n}\n\nTemplate(ast: List(parser.AST))\n\n\nSynonyms:\nhandles.Template\nhandles Template","url":"handles.html#Template"},{"doc":"handles","title":"prepare","content":"pub fn prepare(\n  template: String,\n) -&gt; Result(Template, HandlesError)\n\nSynonyms:\nhandles.prepare\nhandles prepare","url":"handles.html#prepare"},{"doc":"handles","title":"run","content":"pub fn run(\n  template: Template,\n  ctx: Dynamic,\n) -&gt; Result(String, HandlesError)\n\nSynonyms:\nhandles.run\nhandles run","url":"handles.html#run"},{"doc":"handles","title":"handles","content":"","url":"handles.html"},{"doc":"handles/format","title":"format_lex_error","content":"pub fn format_lex_error(\n  error: LexError,\n  template: String,\n) -&gt; String\n\nSynonyms:\nhandles/format.format_lex_error\nhandles/format format_lex_error","url":"handles/format.html#format_lex_error"},{"doc":"handles/format","title":"format_syntax_error","content":"pub fn format_syntax_error(\n  error: SyntaxError,\n  template: String,\n) -&gt; String\n\nSynonyms:\nhandles/format.format_syntax_error\nhandles/format format_syntax_error","url":"handles/format.html#format_syntax_error"},{"doc":"handles/format","title":"handles/format","content":"","url":"handles/format.html"}]);